require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'print_players'
require_relative 'treasure_trove'
require 'csv'

module StudioGame
  class Game
    attr_reader :title

    def initialize(title)
      @title = title
      @players = []
    end

    def load_players(from_file)
      # File.readlines(from_file).each do |line|
      #   player = Player.parse(line)
      #   add_player(player)
      # end
      CSV.foreach(from_file) do |line|
        player = Player.new(line[0], line[1].to_i)
        add_player(player)
      end
    end

    def save_high_scores(to_file = File.join(File.dirname(__FILE__), 'highscores.csv'))
      File.open(to_file, "w") do |file|
        file.puts "#{@title} High Scores:"
        @players.sort.each { |player| file.puts player.format_score }
      end
    end

    def add_player(player)
      @players << player
    end

    def play(rounds)
      puts "There are #{@players.size} players in #{@title}:"

      @players.each do |player|
        puts player
      end

      treasures = TreasureTrove::TREASURES
      puts "\nThere are #{treasures.size} treasures to be found:"
      treasures.each { |treasure| puts "A #{treasure.name} is worth #{treasure.points} points" }

      1.upto(rounds) do |round|
        if block_given?
          break if yield
        end
        puts "\nRound #{round}:"
        @players.each do |player|
          GameTurn.take_turn(player)
        end
      end
    end

    def total_points
      @players.reduce(0) { |sum, player| sum + player.points }
    end

    def print_stats
      strong_players, wimpy_players = @players.partition { |player| player.strong? }
      puts "\n#{@title} Statistics:"

      puts "\n#{strong_players.size} strong players:"
      PrintPlayers.print(strong_players)

      puts "\n#{wimpy_players.size} wimpy players:"
      PrintPlayers.print(wimpy_players)

      puts "\n#{@title} High Scores:"
      @players.sort.each { |player| puts player.format_score }

      @players.sort.each do |player|
        puts "\n#{player.name}'s point totals:"
        player.each_found_treasure do |treasure|
          puts "#{treasure.points} total #{treasure.name} points"
        end
        puts "#{player.points} grand total points"
      end

      puts "\n#{total_points} total points from treasures found"
    end
  end
end